var VizElement = (function () {
    'use strict';
    /**
     * @description Create a VizElement.
     * @typedef VizElement Object used to configure Viz
     * @param {{width:Number, height:Number, nodeShape:String, engine:String, format:String, dotElement:String}} [configObject] Configuration object.
     * @constructor
     */
    function VizElement (configObject) {
        /**
         * @description Desired width of the graph.
         * @type Number
         * @private
         */
        var _width       = configObject.width || undefined;
        /**
         * @description Desired height of the graph.
         * @type Number
         * @private
         */
        var _height      = configObject.height || undefined;
        /**
         * @description Shape of the nodes:
         *              - box, rect, rectangle, square, parallelogram, trapezium, invtrapezium, Msquare
         *              - ellipse, oval, circle, point, egg, doublecircle, Mcircle
         *              - polygon, triangle, diamond, house, pentagon, hexagon, septagon, octagon, doubleoctagon, tripleoctagone, invtriangle, invhouse
         *              - plaintext, plain, note, tab, folder, box3d, component, signature, star, none, underline, promoter, cds, terminator, utr, primersite, restrictionsite, fivepoverhang, threepoverhang, noverhang, assembly, insulator, ribosite, rnastab, proteasesite, proteinstab, rpromoter, rarrow, larrow, lpromoter
         * @type String
         * @private
         */
        var _node_shape  = configObject.nodeShape || undefined;
        /**
         * @description Engine used by Viz - layout for the graph:
         *              - circo, dot, neato, osage, patchwork, twopi, fdp
         * @type String
         * @private
         */
        var _engine      = configObject.engine || "neato";
        /**
         * @description The format to export the data.
         * @type {String}
         * @private
         */
        var _format      = configObject.format || "plain";
        /**
         * @description This is the graph content.
         * @type {String}
         * @private
         */
        var _dot_element = configObject.dotElement || "";
        /**
         * @description Contains the Viz(...) result.
         * @type Viz
         * @private
         */
        var _viz;
        /**
         * @description Contains the graph sizes and the positions and sizes of each nodes.
         * @type {{graph:{width:Number, height:Number, scale:Number}, nodes:{node_Id:{width:Number, height:Number, x:Number, y:Number}}}|null}
         * @private
         */
        var _position = null;

        /**
         * @description Set up the desired size of the graph.
         * @param {Number} width
         * @param {Number} height
         */
        this.setSize = function (width, height) { _width = width ; _height = height; };
        /**
         * @description Give the desired size of the graph.
         * @returns {{width:Number, height:Number}}
         */
        this.getSize = function () { return {width: _width, height : _height}; };
        /**
         * @description Set up the node's shape.
         * @param {String} nodeShape
         */
        this.setNodeShape = function (nodeShape) { _node_shape = nodeShape; };
        /**
         * @description Give the node's shape.
         * @returns {String}
         */
        this.getNodeShape = function () { return _node_shape; };
        /**
         * @description Set up the graph engine (this is the layout to use).
         * @param {String} engine
         */
        this.setEngine = function (engine) { _engine = engine; };
        /**
         * @description Give the graph engine (this is the layout to use).
         * @returns {String}
         */
        this.getEngine = function () { return _engine; };
        /**
         * @description Set up the format to export the data.
         * @param format
         */
        this.setFormat = function (format) { _format = format; };
        /**
         * @description Give the format of the exported data.
         * @returns {String}
         */
        this.getFormat = function () { return _format; };
        /**
         * @description Set up the graph content.
         * @param {String} dotElement
         */
        this.setElement = function (dotElement) { _dot_element = dotElement; };
        /**
         * @description Give the graph content.
         * @returns {String}
         */
        this.getElement = function () { return _dot_element; };
        /**
         * @description Give the graph sizes and the positions and sizes of each nodes.
         * @returns {{graph: {width: Number, height: Number, scale: Number}, nodes: {node_Id: {width: Number, height: Number, x: Number, y: Number}}}}
         */
        this.getPosition = function () {
            if (_position === null) {
                takePosition();
            }
            return _position;
        };

        /**
         * @description Call Viz(...) with the configuration already specified.
         */
        this.run = function () {
            _viz = Viz(getCompleteGraph(), {format:_format, engine:_engine});
            _position = null;
        };
        /**
         * @description Give the complete dot graph with the configuration.
         * digraph G {
         *  [global options]
         *  list of nodes
         *  list of edges
         * }
         * @returns {String}
         */
        function getCompleteGraph () {
            var dotGraphe = "digraph G {\n";

            if (_width || _height) {
                if (_width && _height)
                    dotGraphe += "size =\"" + _width.toString().replace(",", ".") + "," + _height.toString().replace(",", ".") + "!\";\n";
                else
                    dotGraphe += "size = \"" + (_width || _height) + "!\";\n";
            }
            if (_engine === "dot") dotGraphe += "rankdir=\"TB\";\n";
            if (_node_shape) dotGraphe += "node[shape=" + _node_shape + "];\n";

            dotGraphe += _dot_element;
            dotGraphe += "}";

            return dotGraphe;
        }
        /**
         * @description Tell if this line is a graph's line.
         * @param {String} type
         * @returns {boolean}
         */
        function isGraph (type) {
            return type === "graph";
        }
        /**
         * @description Tell if this line is a node's line.
         * @param {String} type
         * @returns {boolean}
         */
        function isNode (type) {
            return type === "node";
        }
        /**
         * @description Tell if this line is a edge's line.
         * @param {String} type
         * @returns {boolean}
         */
        function isEdge (type) {
            return type === "edge";
        }
        /**
         * @description Take the positions and sizes of each nodes. Must be called after the first this.run().
         */
        function takePosition () {
            _position = {graph:{}, nodes:{}};
            new StringIterator(_viz, "\n").forEach(function (line) {
                var wordIterator = new StringIterator(line);
                var type = wordIterator.first();
                var insertIntoAttribute, word;
                if (isGraph(type)) {
                    insertIntoAttribute = {1:"scale", 2:"width", 3:"height"};
                    for (var ig = 1 ; ig <= 3 ; ig++) {
                        word = wordIterator.next();
                        _position.graph[insertIntoAttribute[ig]] = word;
                    }
                } else if (isNode(type)) {
                    insertIntoAttribute = {2:"x", 3:"y", 4:"width", 5:"height"};
                    var node_id = wordIterator.next().replace(/"/g, "");
                    _position.nodes[node_id] = {};
                    for (var iw = 2 ; iw <= 5 ; iw++) {
                        word = wordIterator.next();
                        _position.nodes[node_id][insertIntoAttribute[iw]] = word;
                    }
                }
            }, function (percentage) { console.log("Getting positions: " + percentage.toString().substring(0,4) + "% clear."); });
        }
    }

    return VizElement;
}());